package start6_1;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.Scanner;

/**
 * Author: Josiah Yoder et al.
 * This is just copied from example3_2/Example2.java
 */
public class NullExample implements Runnable {

    private final long BUSY_WAIT_TIME = 1000L;
    private int[] data = new int[1000];

    public void fight() {
        for(int i = 0; i<data.length; i++) {
            // According to the Java spec
            // http://www.cs.cornell.edu/andru/javaspec/17.doc.html
            //
            // when the program reaches this point, it will attempt to obtain a lock
            // on "data"...
            //
            // So any block locking on data (e.g. both synchronized (data) blocks here)
            // will wait for data to be unlocked
            //
            // TLDR: Even though we set data to null, this won't crash the program,
            // because no-one in a syncrhonized block will ever see data as null.
            synchronized (data) {
                int[] tmp = data;
                data = null;

                for(int indBusyWait=0;indBusyWait<BUSY_WAIT_TIME;indBusyWait++) {
                    // do nothing
                }
                data = tmp;
            }

            synchronized (data) {
                int tmp = data[i];
                for(long indBusyWait=0; indBusyWait < BUSY_WAIT_TIME;
                    indBusyWait++) {
                    // do nothing;
                }
                data[i] = tmp + i*i;
            }
        }
    }

    public void check() {
        for(int i = 0; i<data.length; i++) {
            if(data[i] != i*i*2) {
                throw new RuntimeException("I found an error! Element "+i+" " +
                        " should be "+i*i*2+" but is "+data[i]);
            }
        }
        System.out.println("Runnable: Goodbye!");
    }

    @Override
    public void run() {
        fight();
//        queryUser(); // ANOTHER POSSIBLITY FOR THE THREAD.
    }

    public void queryUser() {
        System.out.println("I (runnable) am waiting for you to enter something.");
        Scanner in = new Scanner(System.in);
        String word = in.next();
        System.out.println("Runnable: You entered: "+word);
    }

    public static void main(String[] args) {
        NullExample example = new NullExample();
        Thread thread = new Thread(example, "Action Thread 1");
        Thread thread2 = new Thread(example, "Action Thread 2");

        thread.start();
        thread2.start();

        try {
            thread.join();
            thread2.join();
        } catch (InterruptedException e) {
            System.out.println("main: We were interrupted.");
        }
        example.check();
        System.out.println("main: I'm done! Goodbye!");


        try {
            new PrintWriter(new File("nowhere"));
        } catch (FileNotFoundException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }


}
