package start6_1.squares;

import javax.swing.JPanel;
import javax.swing.Timer;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.ConcurrentModificationException;
import java.util.List;
import java.util.Random;

// from
// http://stackoverflow.com/questions/9713432/drawing-simple-rectangles-on-a-jframe-in-java
public class Squares extends JPanel {
    private final int preferredWidth;
    private final int preferredHeight;

    private static int MAX_JIGGLE_DIST = 3; // measured in pixels

    private List<Rectangle> squares = new ArrayList<Rectangle>();
    private List<Color> colors = new ArrayList<Color>();
    private Random generator = new Random();

    private Timer timer1;

    public Squares(int preferredWidth, int preferredHeight) {
        this.preferredHeight = preferredHeight;
        this.preferredWidth = preferredWidth;
        initTimer1();
        // TODO: APPROACH B
        setPreferredSize(new Dimension(preferredWidth, preferredHeight));
        // TODO: APPROACH C
        setSize(new Dimension(preferredWidth, preferredHeight));
    }

    private void initTimer1() {
        int delayMS1 = 17; // roughly 60 fps.
        ActionListener timer1Listener = new ActionListener() {
            int count = 0;
            @Override
            public synchronized void actionPerformed(ActionEvent e) {
//                try {
                    count++;
                    for(Rectangle square : squares) {
                        Point loc = square.getLocation();
                        // Although getX and getY return doubles, they are stored as ints internally
                        // so we just to cast them back to ints promptly.
                        int newX = (int)loc.getX() + generator.nextInt(2*MAX_JIGGLE_DIST+1)-MAX_JIGGLE_DIST;
                        int newY = (int)loc.getY() + generator.nextInt(2*MAX_JIGGLE_DIST+1)-MAX_JIGGLE_DIST;
                        square.setLocation(new Point(newX, newY));
                    }

                    repaint();
//                } catch(ConcurrentModificationException ignored) {
//                    System.out.println("Caught it here!");
//                }
            }
        };
        timer1 = new Timer(delayMS1,timer1Listener);
        startSquares();
    }

    /**
     *  Starts the timer to get the squares moving.
     */
    private void startSquares() {
        timer1.start();
    }

    private Color randomColor(){
        int red = generator.nextInt(255);
        int green = generator.nextInt(255);
        int blue = generator.nextInt(255);

        try {
            return new Color(red, green, blue);
        }
        catch (IllegalArgumentException e) {
            System.out.println("rgb: "+red+" "+green+" "+blue);
            throw e;
        }
    }

    public void addSquare(int x, int y, int width, int height) {
        Rectangle rect = new Rectangle(x, y, width, height);
        squares.add(rect);
        colors.add(randomColor());
    }

    // TODO: Approach A
    @Override
    public Dimension getPreferredSize() {
        return new Dimension(preferredWidth, preferredHeight);
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;
        for (int i = 0; i<squares.size(); i++) {
            Rectangle rect = squares.get(i);
            Color color = colors.get(i);
            g2.setColor(color);
            g2.draw(rect);
        }
    }
}
